# author: Chantal Peeters
# date: 02.01.2013

### C = Compound Poisson Sum with Gamma distributed summands
### Comparison B: p 0.5, x variable

source('ExaktGammaZ.R');

# Saddlepoints
p = 0.5;
a = 2;
b = 1;
n = 100000;
genauigkeit = 10^(-14);

x = seq(0.15,40,by = 0.15);
saddlepoints = rep(0,length(x))
p_Z = rep(0,length(x))
accuracy = 10^(-14);

for (m in 1:length(x)){
    v = seq(-1,0.87,by = 0.005)
    K1_C = rep(0,length(v));
    i=1;
    while(i <= length(v) &&(i==1 || K1_C[i-1]<x[m])){
      M_Y = b^a/(b-v[i])^a;
      M1_Y = (a* b^a)/((b-v[i])^(a+1)) ;
      K1_C[i] = ((1-p)*M1_Y)/(1-(1-p)*M_Y);
      i=i+1;
    }
    
    j=i-1;
    if(abs(K1_C[j]-x[m])>abs(K1_C[j+1]-x[m])){
      j=j+1;
    }
    SP = v[j]; # starting value
    
    # Newton iteration
    differenz = accuracy +1;
    while(differenz > accuracy){
      M_Y = b^a/(b-SP)^a;
      M1_Y = (a* b^a)/((b-SP)^(a+1)) ;
      M2_Y = a* (a+1)* b^a/(b-SP)^(a+2);
      
      K_C = log(p/(1-(1-p)*M_Y));
      K1_C = ((1-p)*M1_Y)/(1-(1-p)*M_Y);
      K2_C = ((1-p)*M2_Y)/(1-(1-p)*M_Y) + (1-p)^2*M1_Y^2 / (1-(1-p)*M_Y)^2
      
      SP = SP - (K1_C-x[m])/K2_C;
      differenz = abs((K1_C-x[m])/K2_C);
    }
    saddlepoints[m] = SP

  p_Z[m] = ExaktGammaZ(x[m],a,b,p,10^(-17));
}

# values for upper axis
p_r = round(p_Z, digits = 2);
p_s = signif(p_Z, digits = 2);
p_axis_x =c(x[5],x[12],x[19],x[27],x[37],x[54],x[102],x[146],x[187]);
p_axis_p = c(p_r[5],p_r[12],p_r[19],p_r[27],p_r[37],p_r[54],p_r[102],p_s[146],p_s[187]);


## Graph 
par(mfrow=c(1,1),mar=c(5.7,4.1,6.1,2.1));
plot(x,saddlepoints,type="l",lty=2,cex.axis=1.2,cex.lab=1.3,
     ylab="saddlepoint",mgp=c(2.5,0.5,0),xlim=c(0,30),ylim=c(-0.4,0.5),col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.2,mgp=c(1.5,0.5,0));
mtext(side=3,line=1.6,cex=1.3,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.2, cex=1.8,"Saddlepoints",font=2);
dev.copy2pdf(file="A_Sattelpunkte.pdf",width=8,height=5.5)
dev.copy2eps(file="A_Sattelpunkte.eps",width=8,height=5.5)
dev.off()