# author: Chantal Peeters
# date: 02.01.2013

ApproximationGamma<- function(x,a,b,p,accuracy){
  # Computes the sensitivity of P(C>x) for C = "Compound Geometric Sum with 
  # Gamma distributed summands" with the saddlepoint approximation.
  # The saddlepoint cannot be computed directly. Therefore the saddlepoint is 
  # computed with the Newton method. The iteration stops if 
  # the difference between the iteration steps is less than the accuracy.
  
  # Args:
  #    - x
  #    - a, b: parameters of the gamma distribution
  #    - p: parameter of the geometric distribution
  #    - accuracy

	# starting value for the Newton iteration
  # If saddlepoint = 0, the mean of G_C_dlambda for x = 0.01 and -0.01 is taken
  
	if (round((1-p)*a/(b-(1-p)),digits=10)==round(x,digits=10)){
		x = c(x-0.01,x+0.01);
	}

	SP = rep(0,length(x));
	G_C_dp = rep(0,length(x));
	for( k in 1:length(x)){
		v = seq(-1,0.87,by = 0.005)
		K1_C = rep(0,length(v));
		i=1;
			while(i <= length(v) &&(i==1 || K1_C[i-1]<x[k])){
				M_Y = b^a/(b-v[i])^a;
				M1_Y = (a* b^a)/((b-v[i])^(a+1)) ;
				K1_C[i] = ((1-p)*M1_Y)/(1-(1-p)*M_Y);
				i=i+1;
			}
	
		j=i-1;
		if(abs(K1_C[j]-x[k])>abs(K1_C[j+1]-x[k])){
			j=j+1;
		}
		SP[k] = v[j]; # starting value

    # Newton iteration
		difference = accuracy +1;
		while(difference > accuracy){
			M_Y = b^a/(b-SP[k])^a;
			M1_Y = (a* b^a)/((b-SP[k])^(a+1)) ;
			M2_Y = a* (a+1)* b^a/(b-SP[k])^(a+2);

			K_C = log(p/(1-(1-p)*M_Y));
			K1_C = ((1-p)*M1_Y)/(1-(1-p)*M_Y);
			K2_C = ((1-p)*M2_Y)/(1-(1-p)*M_Y) + (1-p)^2*M1_Y^2 / (1-(1-p)*M_Y)^2

			SP[k] = SP[k] - (K1_C-x[k])/K2_C;
			difference = abs((K1_C-x[k])/K2_C);
		}
    
	# saddlepoint approximation
	M_Y = b^a/(b-SP[k])^a;
	M1_Y = (a* b^a)/((b-SP[k])^(a+1)) ;
	M2_Y = a* (a+1)* b^a/(b-SP[k])^(a+2);

	K_C = log(p/(1-(1-p)*M_Y));
	K1_C = ((1-p)*M1_Y)/(1-(1-p)*M_Y);
	K2_C = ((1-p)*M2_Y)/(1-(1-p)*M_Y) + (1-p)^2*M1_Y^2 / (1-(1-p)*M_Y)^2;

	SP_dp =  (x[k]*M_Y+M1_Y)/((1-p)*(M2_Y+x[k]*M1_Y));

	M_Y_dp = M1_Y * SP_dp;
	M1_Y_dp = M2_Y * SP_dp;
	M2_Y_dp =  a* (a+1)* (a+2)* b^a/(b-SP[k])^(a+3)* SP_dp;

	K_C_dp = 1/p - (M_Y-(1-p)*M_Y_dp)/(1-(1-p)*M_Y);
	K1_C_dp = (-M1_Y+(1-p)*M1_Y_dp)/(1-(1-p)*M_Y) - (1-p)*M1_Y*(M_Y-(1-p)*M_Y_dp)/(1-(1-p)*M_Y)^2;
	K2_C_dp = (-M2_Y+(1-p)*M2_Y_dp)/(1-(1-p)*M_Y) - (1-p)*M2_Y*(M_Y-(1-p)*M_Y_dp)/(1-(1-p)*M_Y)^2 + 2*(1-p)*M1_Y*(-M1_Y +(1-p)*M1_Y_dp)/(1-(1-p)*M_Y)^2 -2*(1-p)^2*M1_Y^2*(M_Y-(1-p)*M_Y_dp)/(1-(1-p)*M_Y)^3;


	v_x = SP[k] * sqrt(K2_C);
	r_x = sign(SP[k]) * sqrt(2*(SP[k] * K1_C - K_C));

	v_x_dp = SP_dp * sqrt(K2_C) + SP[k] * (K2_C_dp )/(2*sqrt(K2_C));
	r_x_dp = 1/r_x * (SP_dp *K1_C + SP[k] * (K1_C_dp) - K_C_dp);

	G_C_dp[k] = -dnorm(r_x,mean=0,sd=1)* ( r_x /v_x*r_x_dp  +  (-1/(r_x)^2 *r_x_dp + 1/(v_x)^2 *v_x_dp));
	}
	return(G_C_dp=mean(G_C_dp));
}
