# author: Chantal Peeters
# date: 02.01.2013

### C = Compound Poisson Sum with Gamma distributed summands
### Comparison B: p variable, x = 10.25

# Saddlepoints
x = 14;
a = 2;
b = 1;
accuracy = 10^(-12);

p = seq(0.01,0.99,by=0.005)
saddlepoints = rep(0,length(p))

for (m in 1:length(p)){ 
    v = seq(-1,0.87,by = 0.005)
    K1_C = rep(0,length(v));
    i=1;
    while(i <= length(v) &&(i==1 || K1_C[i-1]<x)){
      M_Y = b^a/(b-v[i])^a;
      M1_Y = (a* b^a)/((b-v[i])^(a+1)) ;
      K1_C[i] = ((1-p[m])*M1_Y)/(1-(1-p[m])*M_Y);
      i=i+1;
    }
    
    j=i-1;
    if(abs(K1_C[j]-x)>abs(K1_C[j+1]-x)){
      j=j+1;
    }
    SP = v[j]; # starting value
    
    # Newton iteration
    differenz = accuracy +1;
    while(differenz > accuracy){
      M_Y = b^a/(b-SP)^a;
      M1_Y = (a* b^a)/((b-SP)^(a+1)) ;
      M2_Y = a* (a+1)* b^a/(b-SP)^(a+2);
      
      K_C = log(p[m]/(1-(1-p[m])*M_Y));
      K1_C = ((1-p[m])*M1_Y)/(1-(1-p[m])*M_Y);
      K2_C = ((1-p[m])*M2_Y)/(1-(1-p[m])*M_Y) + (1-p[m])^2*M1_Y^2 / (1-(1-p[m])*M_Y)^2
      
      SP = SP - (K1_C-x)/K2_C;
      differenz = abs((K1_C-x)/K2_C);
    }
    saddlepoints[m] = SP
}
  




## Graph
par(mfrow=c(1,1),mar=c(5.7,4.1,6.1,2.1));
plot(p,saddlepoints,type="l",lty=2,cex.axis=1.2,cex.lab=1.3,
     ylab="Sattelpunkt",mgp=c(2.5,0.5,0),xlim=c(0,1),col="blue",lwd=0.5);
dev.copy2pdf(file="B_Sattelpunkte.pdf",width=8,height=5.5)
dev.copy2eps(file="B_Sattelpunkte.eps",width=8,height=5.5)
dev.off()