# author: Chantal Peeters
# date: 02.01.2013

SimulationGamma_Masswechsel<- function(x,a,b,p,n,accuracy){  
  # Computes the sensitivity of P(C>x) for C = "Compound Geometric Sum with Gamma 
  # distributed summands" with the Monte Carlo Method ISSEM.
  # Since infinite sums cannot be calculated with R, the first n summands are summed up, 
  # till the summand is bigger than the specified accuracy.
  
  # Args:
  #    - x
  #    - a, b: parameters of the gamma distribution
  #    - p: parameter of the geometric distribution
  #    - accuracy
  
	s = matrix(0,nrow = n);
	Z = matrix(0,nrow = n);
	N = matrix(0,nrow = n);
	C = matrix(0,nrow = n);

	# change of measure only if x > a/b*(1-p)/p
	if(x > (a/b*(1-p)/p)){
		v = seq(-1,0.87,by = 0.005)
		K1_C = rep(0,length(v));
		i=1;
			while(i <= length(v) &&(i==1 || K1_C[i-1]<x)){
				M_Y = b^a/(b-v[i])^a;
				M1_Y = (a* b^a)/((b-v[i])^(a+1)) ;
				K1_C[i] = ((1-p)*M1_Y)/(1-(1-p)*M_Y);
				i=i+1;
			}

	
		j=i-1;
		SP = v[j];

		summand = accuracy +1;
		while(summand > accuracy){
			M_Y = b^a/(b-SP)^a;
			M1_Y = (a* b^a)/((b-SP)^(a+1)) ;
			M2_Y = a* (a+1)* b^a/(b-SP)^(a+2);

			K_C = log(p/(1-(1-p)*M_Y));
			K1_C = ((1-p)*M1_Y)/(1-(1-p)*M_Y);
			K2_C = ((1-p)*M2_Y)/(1-(1-p)*M_Y) + (1-p)^2*M1_Y^2 / (1-(1-p)*M_Y)^2

			SP = SP - (K1_C-x)/K2_C;
			summand = abs((K1_C-x)/K2_C);
		}

		theta = SP; # E_{theta}[C] = x
			
		M_Y = b^a/(b-theta)^a;
		K_C = log(p/(1-(1-p)*M_Y));


		p_theta = 1-(1-p)*M_Y;
		b_theta = b - theta;
	}else{
		p_theta = p;
		b_theta = b;
	}
 
	for (i in 1:n){
		N[i] = rgeom(n=1,prob=p_theta);
		if(N[i] != 0){
			C[i] = rgamma(n= 1,shape =N[i]*a,scale=1/b_theta);
			if (C[i] > x){ 
				Z[i]=1;
			}
			s[i] = 1/p-N[i]/(1-p);
		}
	}

	if(x >  (a/b*(1-p)/p)){
		sensitivity = mean(s*Z*exp(-theta*C + K_C));		
	}else{
		sensitivity = mean(s*Z);
	}

	return(sensitivity = sensitivity);
}
