# author: Chantal Peeters
# date: 02.01.2013

### C = Compound Poisson Sum with Gamma distributed summands
### Comparison A: lambda = 1, x variable

source('ExaktGammaZ.R');

# Saddlepoints
lambda = 1;
alpha = 2;
beta = 1;

x = seq(0.1,20,by = 0.1)
saddlepoints = rep(0,length(x))
p = rep(0,length(x))
accuracy = 10^(-10);

for (i in 1:length(x)){
  saddlepoints[i] = beta - (lambda * alpha * beta^alpha/x[i])^(1/(alpha+1));
  p[i] = ExaktGammaZ(x[i],alpha,beta,lambda,accuracy);
}

# values for upper axis
p_r = round(p, digits = 2);
p_s = signif(p, digits = 2);
p_axis_x =c(x[5],x[12],x[19],x[27],x[37],x[54],x[102],x[146],x[187]);
p_axis_p = c(p_r[5],p_r[12],p_r[19],p_r[27],p_r[37],p_r[54],p_r[102],p_s[146],p_s[187]);


## Graph 
par(mfrow=c(1,1),mar=c(5.7,4.1,6.1,2.1));
plot(x,saddlepoints,type="l",lty=2,cex.axis=1.2,cex.lab=1.3,
     ylab="saddlepoint",mgp=c(2.5,0.5,0),xlim=c(0,20),ylim=c(-0.4,0.5),col="blue",lwd=0.5);
abline(h=0,lwd=0.5);
axis(3,at=p_axis_x,labels=p_axis_p,cex.axis=1.2,mgp=c(1.5,0.5,0));
mtext(side=3,line=1.6,cex=1.3,expression(bold(P)(C[N]>x)));
mtext(side=3, line=3.2, cex=1.8,"Saddlepoints",font=2);
dev.copy2pdf(file="A_Sattelpunkte.pdf",width=8,height=5.5)
dev.copy2eps(file="A_Sattelpunkte.eps",width=8,height=5.5)
dev.off()