# author: Chantal Peeters
# date: 02.01.2013

ApproximationGamma2<- function(x,a,b,lambda){
  # Computes the sensitivity of P(C>x) for C = "Compound Poisson Sum with Gamma distributed 
  # summands" with the saddlepoint approximation.
  
  # Args:
  #    - x
  #    - a, b: parameters of the gamma distribution
  #    - lambda: parameter of the Poisson distribution
  
	SP = b - (lambda * a * b^a/x)^(1/(a+1));
	
	# If saddlepoint = 0, the mean of G_C_dlambda forx = 0.01 and -0.01 is taken
	if (SP == 0){
		x = c(x-0.01,x+0.01);
		SP = c(b - (lambda * a * b^a/x[1])^(1/(a+1)),b - (lambda * a * b^a/x[2])^(1/(a+1)));
	}
	
	G_C_dlambda = rep(0,length(SP));
	for(i in 1:length(SP)){
		M_Y = b^a/(b-SP[i])^a;
		M1_Y = (a* b^a)/((b-SP[i])^(a+1)) ;
		M2_Y = a* (a+1)* b^a/(b-SP[i])^(a+2);
		M3_Y = a* (a+1)*(a+2)* b^a/(b-SP[i])^(a+3);

		SP_dlambda = -M1_Y/(lambda*M2_Y);
			
		K_C = lambda*(M_Y-1);
		K1_C = lambda*M1_Y;
		K2_C = lambda*M2_Y;

		K_C_dlambda = M_Y -1 + lambda*M1_Y*SP_dlambda;
		K2_C_dlambda = M2_Y + lambda*M3_Y*SP_dlambda;
	
		v_x = SP[i] * sqrt(K2_C);
		r_x = sign(SP[i]) * sqrt(2*(SP[i] *K1_C - K_C));

		v_x_dlambda = SP_dlambda * sqrt(K2_C) + SP[i] * K2_C_dlambda/(2*sqrt(K2_C));
		r_x_dlambda = 1/r_x * (SP_dlambda * K1_C - K_C_dlambda);

		G_C_dlambda[i] = -dnorm(r_x,mean=0,sd=1)* ( r_x /v_x*r_x_dlambda  +  (-1/(r_x)^2 *r_x_dlambda + 1/(v_x)^2 *v_x_dlambda));
	}
	
	return(G_C_dlambda=mean(G_C_dlambda));
}
