# author: Chantal Peeters
# date: 02.01.2013

SimulationGamma_Masswechsel<- function(x,a,b,lambda,n){
  # Computes the sensitivity of P(C>x) for C = "Compound Poisson Sum with Gamma distributed 
  # summands" with the Monte Carlo Method ISSEM.
  
  # Args:
  #    - x
  #    - a, b: parameters of the gamma distribution
  #    - lambda: prameter of the Poisson distribution
  #    - n: simulation parameter
  
	s = matrix(0,nrow = n);
	Z = matrix(0,nrow = n);
	N = matrix(0,nrow = n);
	C = matrix(0,nrow = n);

	# change of measure only if x > 2*lambda

	if(x > (2*lambda)){
		SP = b - (lambda * a * b^a/x)^(1/(a+1));
		theta = SP; # E_{theta}[C] = x
		M_Y = b^a/(b-theta)^a;

		lambda_theta = lambda*M_Y;
		b_theta = b - theta;
	}else{
		lambda_theta = lambda;
		b_theta = b;
	}
 
	for (i in 1:n){
		N[i] = rpois(n=1,lambda = lambda_theta);
		if(N[i] != 0){
			C[i] = rgamma(n= 1,shape =N[i]*a,scale=1/b_theta);
			if (C[i] > x){ 
				Z[i]=1;
			}
			s[i] = N[i]/lambda - 1;
		}
	}

	if(x > (2*lambda)){
		sensitivity = mean(s*Z*exp(-theta*C + lambda*(M_Y - 1)));		
	}else{
		sensitivity = mean(s*Z);
	}

	return(sensitivity = sensitivity);
}
