# author: Chantal Peeters
# date: 02.01.2013

source("M_Y.R");
source("M1_Y.R");
source("M2_Y.R");
source("M3_Y.R");

ApproximationWeibull<- function(x,c,lambda,accuracy){
  # Computes the sensitivity of P(C>x) for C = "Compound Poisson Sum with Weibull 
  # distributed summands" with the saddlepoint approximation.
  # The saddlepoint cannot be computed directly. Therefore the saddlepoint is 
  # computed with the Newton method. The iteration stops if 
  # the difference between the iteration steps is less than the accuracy.
  
  # Args:
  #    - x
  #    - c: parameter of the Weibull distribution
  #    - lambda: parameter of the Poisson distribution
  #    - accuracy 
  
  # If saddlepoint = 0, the mean of G_C_dlambda for x = 0.0001 and -0.0001 is taken 
	if( x== gamma(1+1/c)*lambda){ 
		SP = c(0.0001,-0.0001);
	}
	else{	
		# starting value for the Newton iteration
		SP = log(x/lambda);
		difference = accuracy +1;
    
		# Newton iteration
		while(difference > accuracy){
			SP_alt = SP;
			SP = SP - (lambda*M1_Y(SP,c,accuracy)-x)/(lambda*M2_Y(SP,c,accuracy));
			difference = abs(SP-SP_alt);
		}
	}
	
  # saddlepoint approximation
	G_C_dlambda = rep(0,length(SP));
	for(i in 1:length(SP)){
		M_Y = M_Y(SP[i],c,accuracy);
		M1_Y = M1_Y(SP[i],c,accuracy);
		M2_Y = M2_Y(SP[i],c,accuracy);
		M3_Y = M3_Y(SP[i],c,accuracy);

		SP_dlambda = - M1_Y/(lambda*M2_Y);

		M_Y_dlambda = M1_Y * SP_dlambda;
		M1_Y_dlambda = - x * lambda^(-2);
		M2_Y_dlambda =  M3_Y * SP_dlambda;

		v_x = SP[i] * sqrt(lambda * M2_Y);
		r_x = sign(SP[i]) * sqrt(2*(SP[i] * lambda * M1_Y - lambda * M_Y +lambda));

		v_x_dlambda = SP_dlambda * sqrt(lambda *  M2_Y) + SP[i] * (M2_Y+ lambda*M2_Y_dlambda )/(2*sqrt(lambda * M2_Y));
		r_x_dlambda = 1/r_x * (SP_dlambda * lambda* M1_Y + SP[i] * (M1_Y + lambda * M1_Y_dlambda) - M_Y +1 - lambda *M_Y_dlambda);

		G_C_dlambda[i] = -dnorm(r_x,mean=0,sd=1)* ( r_x /v_x*r_x_dlambda  +  (-1/(r_x)^2 *r_x_dlambda + 1/(v_x)^2 *v_x_dlambda));
	}

	return(G_C_dlambda=mean(G_C_dlambda));
}
